Package: DQA
Title: Data Quality Assessment Tools
Version: 0.1.1
Date: 2026-04-18
Authors@R: 
    c(person("Hamed", "Tabesh", email = "tabesh79@gmail.com", role = c("aut")),
      person("Elham", "Hosseinzadeh", email = "elham.hh2012@gmail.com", role = c("aut", "cre")),
      person("Marziyeh", "Afkanpour", email = "maafkanpour@gmail.com", role = c("aut")))
Description: In the context of data quality assessment, this package provides a number of functions for evaluating data quality across various dimensions, including completeness, plausibility, concordance, conformance, currency, timeliness, and correctness. It has been developed based on two well-known frameworks—Michael G. Kahn (2016) <doi:10.13063/2327-9214.1244> and Nicole G. Weiskopf (2017) <doi:10.5334/egems.218>—for data quality assessment. Using this package, users can evaluate the quality of their datasets, provided that corresponding metadata are available.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
Imports: ggplot2, data.table
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-20 06:07:50 UTC; ELHAM
Author: Hamed Tabesh [aut],
  Elham Hosseinzadeh [aut, cre],
  Marziyeh Afkanpour [aut]
Maintainer: Elham Hosseinzadeh <elham.hh2012@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-20 06:20:02 UTC
Built: R 4.6.0; ; 2026-04-28 02:21:24 UTC; unix
