# Generated by roxygen2: do not edit by hand

export(check_missing_itemwise)
export(check_missing_record)
export(check_missing_segments)
export(concordance_check)
export(conformance_check)
export(correctness_check)
export(currency_check)
export(plausible_check)
export(timeliness_check)
importFrom(data.table,as.data.table)
importFrom(data.table,rbindlist)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,coord_polar)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_stack)
importFrom(ggplot2,scale_fill_brewer)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(stats,median)
importFrom(stats,reorder)
importFrom(stats,setNames)
